/**********************************************************************
 * File: contex.cpp
 * Version: 0.1
 * Project: adeos
 * Created: jan 10 01
 * Last modified:
 *
 * Comment: provides dummy functions for 
 * contextInit and contextSwitch
 *
 *
 *
 * Author: Jens-Christian Lache
 * Fachbereich: TI3 Technische Informatik
 * Prof. Dr. Thomas Teufel
 * http://www.ti3.tu-harburg.de/deutsch/
 **********************************************************************/


#include "adeos.h"
#include <stdio.h>
inline int saveContext(Context * pContext);
inline void restoreContext(Context * pContext);

/**********************************************************************
 * Function: contextInit
 * Object: initialize the context field of a new task object
 * created: 17 Jan
 * last modified: 
 **********************************************************************/

void contextInit(Context * context, void (*run)(Task *), Task * task, int * pStackTop) {

  context->pc = (int) run;
  context->lr = 0;
  context->sp = (int) pStackTop;
  context->cpsr = USER_MODE;
  context->taskPointer = (int) task;

  // debug
#ifdef WITH_PRINTF
  printf\
  (" contextInit (Context * context = 0x%X, void (*run)(Task *) = 0x%X, Task * task = 0x%X, int * pStackTop = 0x%X)\n"\
   , (int) context ,(int)  run,(int)  task,(int)  pStackTop);
  printf("context->pc = 0x%X\n", (int) context->pc );
  printf("context->lr = 0x%X\n", (int) context->lr );
  printf("context->sp = 0x%X\n", (int) context->sp );
  printf("context->cpsr = 0x%X\n", (int) context->cpsr );
  printf("context->taskPointer = 0x%X\n", (int) context->taskPointer );
  // end debug
#endif //  WITH_PRINTF

}

/**********************************************************************
 * Function: contextSwitch
 * Object: perform a context switch
 * created: 17 Jan
 * last modified: 
 **********************************************************************/

void contextSwitch(Context * pOldContext, Context * pNewContext) {
  
  if (saveContext(pOldContext)) {
    restoreContext(pNewContext);
    // this line will never be executed!
  }
  
  // Instead, the restored task continues to execute at this point
}


/**********************************************************************
 * Function: saveContext
 * Object: save Context
 * Comment: a task is allways waking up or goes to sleep in this function
 * return: non-zero, if the task goes to sleep
 *         zero, if the task is waking up
 * created: 17 Jan
 * last modified: 
 **********************************************************************/

inline int saveContext(Context * pContext) {
  if (pContext == NULL) {
    return 1;
  }
}

inline void restoreContext(Context * pContext) {
  int volatile pc;
  int volatile lr;
  int volatile sp;
  int volatile cpsr;
  int taskPointer;
  pc = pContext->pc;
  lr = pContext->lr;
  sp = pContext->sp;
  cpsr = pContext->cpsr;
  taskPointer = pContext->taskPointer;
  asm volatile ("mov lr,%0": :"r" (lr));
  asm volatile ("mov sp,%0": :"r" (sp));
  asm volatile ("@ do something with cpsr");
  asm volatile ("mov r0,%0": :"r" (taskPointer):"r0");
  asm volatile ("mov pc,%0": :"r" (pc));
}
