/**********************************************************************
 *
 * Filename:    hello.cpp
 * 
 * Description: A "Hello, World!" application built on top of ADEOS.
 *
 * Notes:       This module is hardware-independent.  All of the 
 *              hardware dependence has been captured in the device
 *              driver and operating system modules.
 *
 * 
 * Copyright (c) 1998 by Michael Barr.  This software is placed into
 * the public domain and may be used for any purpose.  However, this
 * notice must not be changed or removed and no warranty is either
 * expressed or implied by its publication or distribution.
 **********************************************************************/

#include "adeos.h"
#include <stdio.h>

/**********************************************************************
 *
 * Function:    taskAfunction()
 *
 * Description: Blink the red LED ten times a second.
 * 
 * Notes:       This outer loop is hardware-independent.  However, it 
 *              calls the hardware-dependent function toggleLed().
 *
 * Returns:     This routine contains an infinite loop.
 *
 **********************************************************************/
void
taskAfunction(void)
{
  asm volatile ("mov r0, #10": : :"r0");
  asm volatile ("mov r1, #11": : :"r1");
  asm volatile ("mov r2, #12": : :"r2");
  asm volatile ("mov r3, #13": : :"r3");
  asm volatile ("mov r4, #14": : :"r4");
  asm volatile ("mov r5, #15": : :"r5");
  asm volatile ("mov r6, #16": : :"r6");
  asm volatile ("mov r7, #17": : :"r7");
  asm volatile ("mov r8, #18": : :"r8");
  asm volatile ("mov r9, #19": : :"r9");
  
}   /* taskAfunction() */


/**********************************************************************
 *
 * Function:    taskBfunction()
 * 
 * Description: Send a text message to the serial port periodically.
 *
 * Notes:       This outer loop is hardware-independent.  
 *
 * Returns:     This routine contains an infinite loop.
 *
 **********************************************************************/
void
taskBfunction(void)
{
  asm volatile ("mov r0, #30": : :"r0");
  asm volatile ("mov r1, #31": : :"r1");
  asm volatile ("mov r2, #32": : :"r2");
  asm volatile ("mov r3, #33": : :"r3");
  asm volatile ("mov r4, #34": : :"r4");
  asm volatile ("mov r5, #35": : :"r5");
  asm volatile ("mov r6, #36": : :"r6");
  asm volatile ("mov r7, #37": : :"r7");
  asm volatile ("mov r8, #38": : :"r8");
  asm volatile ("mov r9, #39": : :"r9");
}   /* taskBfunction() */

Sched::SchedState  Sched::state = Uninitialized;

int       Sched::interruptLevel = 0;
int       Sched::bSchedule      = 0;

Task *    Sched::pRunningTask = NULL;
TaskList  Sched::readyList;


MemAllocator internalMem;
Sched     os;
//Task  Sched::idleTask(idle, 0, 128);
Task  idleTask(idle, 0, 128);

Task  taskA(taskAfunction, 150, 512);
Task  taskB(taskBfunction, 200, 512);

/*********************************************************************
 * 
 * Function:    main()
 *
 * Description: This function is responsible for starting the ADEOS
 *              scheduler only.
 *
 * Notes:       
 *
 * Returns:     This function will never return!
 *
 *********************************************************************/
void
main(void)
{
    init_swi();
#ifdef DEBUG
    int swiArgument;
    int restoreValue;
  // test the code
  swiArgument = REASON_CHANGE_TO_SV;
  asm volatile ("mov r0, %0": :"r" (swiArgument):"r0");
  asm volatile ("swi");
  // we should be now in SV
  // switch back to user mode
  restoreValue = 0x10;
  asm volatile ("mov r0, %0": :"r" (restoreValue):"r0");
  asm volatile ("msr CPSR, r0");

#endif
    os.start();
    // This point will never be reached.

}   /* main() */
