/**********************************************************************
 * File: memAllocator.cpp
 * Version: 0.1
 * Project: adeos
 * Created: 19 jan 2001
 * Last modified: 
 *
 * Comment:
 * The simulator does not know about the monitor program
 * "angel". "new int[stackSize];" does somewhere make a
 * call to the swi handler of angel. memAllocator will
 * get a decent amount of mem and allocate parts of it
 * to the callers of "memAllocator.new(int );"
 *
 *
 *
 * Author: Jens-Christian Lache
 * Fachbereich: TI3 Technische Informatik
 * Prof. Dr. Thomas Teufel
 * http://www.ti3.tu-harburg.de/deutsch/
 **********************************************************************/

#include "memAllocator.h"
#include "adeos.h"

/**********************************************************************
 *
 * Method:      MemAllocator()
 *
 * Description: Initialize memAllocator
 * - init all elements of mem to "unused"
 * - set pFreeMem to first free mem address
 * Notes:
 * 
 * Returns:     
 *
 **********************************************************************/

MemAllocator::MemAllocator() {
  int i;
  
  for (i=0;i<MAX_MEM_POINTER; i++){
    mem[i].usedFlag=FALSE;
    mem[i].pStart = MEM_END;
    mem[i].pEnd = MEM_END;
  }
  
}

/**********************************************************************
 *
 * Method:      newMem(int memWords)
 *
 * Description: 
 * - 
 * - 
 * Notes:
 * 
 * Returns:     
 *
 **********************************************************************/

int * MemAllocator::newMem(int memWords) {
  int i,memElement,counter;
  int * pFreeMem;
  pFreeMem = MEM_START;
  counter = 0;
  while(counter < MAX_MEM_POINTER) {
    memElement = MAX_MEM_POINTER;
    for(i=counter;i<MAX_MEM_POINTER;i++){
      // is mem[i] not in use ?
      if(mem[i].usedFlag == FALSE) {
	memElement = i; // index of free element
	i = MAX_MEM_POINTER;
      } else { // mem[i] is in use
	pFreeMem = mem[i].pEnd+1;
      }
    }
    // No free element found
    if (memElement == MAX_MEM_POINTER) {
      return NULL;
    }
    if ((pFreeMem+memWords) < mem[memElement+1].pStart) {
      mem[memElement].usedFlag= TRUE;
      mem[memElement].pStart = pFreeMem;
      mem[memElement].pEnd = pFreeMem + memWords;
      return pFreeMem;
    } else {
      counter = memElement;
    }
  }
  // All free elements are to small 
  return NULL;
}

/**********************************************************************
 *
 * Method:      deleteMem(int * pMemorie)
 *
 * Description: 
 * - 
 * - 
 * Notes:
 * 
 * Returns:     
 *
 **********************************************************************/

int MemAllocator::deleteMem(int * pMemorie) {
  int i,memElement;

  for (i=0;i<MAX_MEM_POINTER;i++){
    if (mem[i].pStart == pMemorie)  {
      // Element is found
      // -> free it
      mem[i].usedFlag = FALSE;
      mem[i].pStart = MEM_END;
      mem[i].pEnd = MEM_END;
      return TRUE;
    }
  }
  // The location wasnt found
  return FALSE;
}
